@@ -11,9 +11,10 @@ import logging
 import socket
 import re
 import xml.etree.ElementTree as ET
+from concurrent.futures import ThreadPoolExecutor
 from urllib.parse import urlparse
 import requests
-import ifaddr
+import netifaces
 
 _LOGGER = logging.getLogger('DenonSSDP')
 
@@ -27,8 +28,11 @@ SSDP_ST_3 = "urn:schemas-upnp-org:device:MediaRenderer:1"
 
 SSDP_ST_LIST = (SSDP_ST_1, SSDP_ST_2, SSDP_ST_3)
 
+SSDP_LOCATION_PATTERN = re.compile(r'(?<=LOCATION:\s).+?(?=\r)')
+
 SCPD_XMLNS = "{urn:schemas-upnp-org:device-1-0}"
 SCPD_DEVICE = "{xmlns}device".format(xmlns=SCPD_XMLNS)
+SCPD_DEVICELIST = "{xmlns}deviceList".format(xmlns=SCPD_XMLNS)
 SCPD_DEVICETYPE = "{xmlns}deviceType".format(xmlns=SCPD_XMLNS)
 SCPD_MANUFACTURER = "{xmlns}manufacturer".format(xmlns=SCPD_XMLNS)
 SCPD_MODELNAME = "{xmlns}modelName".format(xmlns=SCPD_XMLNS)
@@ -36,7 +40,10 @@ SCPD_SERIALNUMBER = "{xmlns}serialNumber".format(xmlns=SCPD_XMLNS)
 SCPD_FRIENDLYNAME = "{xmlns}friendlyName".format(xmlns=SCPD_XMLNS)
 SCPD_PRESENTATIONURL = "{xmlns}presentationURL".format(xmlns=SCPD_XMLNS)
 
-DEVICETYPE_DENON = "urn:schemas-upnp-org:device:MediaRenderer:1"
+SUPPORTED_DEVICETYPES = [
+    "urn:schemas-upnp-org:device:MediaRenderer:1",
+    "urn:schemas-upnp-org:device:MediaServer:1",
+    ]
 
 SUPPORTED_MANUFACTURERS = ["Denon", "DENON", "Marantz"]
 
@@ -54,13 +61,12 @@ def ssdp_request(ssdp_st, ssdp_mx=SSDP_MX):
 
 def get_local_ips():
     """Get IPs of local network adapters."""
-    adapters = ifaddr.get_adapters()
     ips = []
-    for adapter in adapters:
-        # pylint: disable=invalid-name
-        for ip in adapter.ips:
-            if isinstance(ip.ip, str):
-                ips.append(ip.ip)
+    # pylint: disable=c-extension-no-member
+    for interface in netifaces.interfaces():
+        addresses = netifaces.ifaddresses(interface)
+        for address in addresses.get(netifaces.AF_INET, []):
+            ips.append(address["addr"])
     return ips
 
 
@@ -71,14 +77,14 @@ def identify_denonavr_receivers():
     Returns a list of dictionaries which includes all discovered Denon AVR
     devices with keys "host", "modelName", "friendlyName", "presentationURL".
     """
-    # Sending SSDP broadcast message to get devices
-    devices = send_ssdp_broadcast()
+    # Sending SSDP broadcast message to get resource urls from devices
+    urls = send_ssdp_broadcast()
 
     # Check which responding device is a DenonAVR device and prepare output
     receivers = []
-    for device in devices:
+    for url in urls:
         try:
-            receiver = evaluate_scpd_xml(device["URL"])
+            receiver = evaluate_scpd_xml(url)
         except requests.exceptions.RequestException:
             continue
         if receiver:
@@ -89,65 +95,65 @@ def identify_denonavr_receivers():
 
 def send_ssdp_broadcast():
     """
-    Send SSDP broadcast message to discover UPnP devices.
+    Send SSDP broadcast messages to discover UPnP devices.
 
-    Returns a list of dictionaries with "address" (IP, PORT) and "URL"
-    of SCPD XML for all discovered devices.
+    Returns a set of SCPD XML resource urls for all discovered devices.
     """
     # Send up to three different broadcast messages
     ips = get_local_ips()
     res = []
-    # pylint: disable=invalid-name
-    for ip in ips:
-        # Ignore 169.254.0.0/16 adresses
-        if re.search("169.254.*.*", ip):
-            continue
-        for i, ssdp_st in enumerate(SSDP_ST_LIST):
-            # Prepare SSDP broadcast message
-            sock = socket.socket(
-                socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
-            sock.settimeout(SSDP_MX)
-            sock.bind((ip, 0))
-            sock.sendto(ssdp_request(ssdp_st), (SSDP_ADDR, SSDP_PORT))
 
-            # Collect all responses within the timeout period
-            try:
-                while True:
-                    res.append(sock.recvfrom(10240))
-            except socket.timeout:
-                sock.close()
+    futures = []
+    with ThreadPoolExecutor() as executor:
+        for ip_addr in ips:
+            futures.append(executor.submit(send_ssdp_broadcast_ip, ip_addr))
 
-            if res:
-                _LOGGER.debug(
-                    "Got results after %s SSDP queries using ip %s", i + 1, ip)
-                sock.close()
-                break
-        if res:
-            break
+        for future in futures:
+            res.extend(future.result())
 
     # Prepare output of responding devices
-    devices = {}
-    device = {}
+    urls = set()
 
     for entry in res:
-        device["address"] = entry[1]
         # Some string operations to get the receivers URL
         # which could be found between LOCATION and end of line of the response
-        en_decoded = entry[0].decode("utf-8")
-        # If location is not found, skip the entry
-        try:
-            device["URL"] = (
-                en_decoded[
-                    en_decoded.lower().index(
-                        "location:") + 10:en_decoded.index(
-                            "\r\n", en_decoded.lower().index("location:"))]
-            )
-        except ValueError:
-            continue
-        devices[device["address"]] = device.copy()
+        entry_text = entry[0].decode("utf-8")
+        match = SSDP_LOCATION_PATTERN.search(entry_text)
+        if match:
+            urls.add(match.group(0))
 
-    _LOGGER.debug("Following devices found: %s", list(devices.values()))
-    return list(devices.values())
+    _LOGGER.debug("Following devices found: %s", urls)
+    return urls
+
+
+def send_ssdp_broadcast_ip(ip_addr):
+    """Send SSDP broadcast messages to a single IP."""
+    res = []
+    # Ignore 169.254.0.0/16 adresses
+    if re.search("169.254.*.*", ip_addr):
+        return res
+    # Prepare SSDP broadcast message
+    sock = socket.socket(
+        socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
+    sock.settimeout(SSDP_MX)
+    sock.bind((ip_addr, 0))
+    for ssdp_st in SSDP_ST_LIST:
+        sock.sendto(ssdp_request(ssdp_st), (SSDP_ADDR, SSDP_PORT))
+
+    # Collect all responses within the timeout period
+    try:
+        while True:
+            res.append(sock.recvfrom(10240))
+    except socket.timeout:
+        pass
+    finally:
+        sock.close()
+
+    if res:
+        _LOGGER.debug(
+            "Got %s results after SSDP queries using ip %s", len(res), ip_addr)
+
+    return res
 
 
 def evaluate_scpd_xml(url):
@@ -174,28 +180,44 @@ def evaluate_scpd_xml(url):
             # Look for manufacturer "Denon" in response.
             # Using "try" in case tags are not available in XML
             device = {}
+            device_xml = None
             device["manufacturer"] = (
                 root.find(SCPD_DEVICE).find(SCPD_MANUFACTURER).text)
 
             _LOGGER.debug("Device %s has manufacturer %s", url,
                           device["manufacturer"])
-            if (device["manufacturer"] in SUPPORTED_MANUFACTURERS and
-                    root.find(SCPD_DEVICE).find(
-                        SCPD_DEVICETYPE).text == DEVICETYPE_DENON):
+
+            if not device["manufacturer"] in SUPPORTED_MANUFACTURERS:
+                return False
+
+            if (root.find(SCPD_DEVICE).find(SCPD_DEVICETYPE).text
+                    in SUPPORTED_DEVICETYPES):
+                device_xml = root.find(SCPD_DEVICE)
+            elif root.find(SCPD_DEVICE).find(SCPD_DEVICELIST) is not None:
+                for dev in root.find(SCPD_DEVICE).find(SCPD_DEVICELIST):
+                    if (dev.find(SCPD_DEVICETYPE).text in SUPPORTED_DEVICETYPES
+                            and dev.find(SCPD_SERIALNUMBER) is not None):
+                        device_xml = dev
+                        break
+
+            if device_xml is None:
+                return False
+
+            if device_xml.find(SCPD_PRESENTATIONURL) is not None:
                 device["host"] = urlparse(
-                    root.find(SCPD_DEVICE).find(
+                    device_xml.find(
                         SCPD_PRESENTATIONURL).text).hostname
                 device["presentationURL"] = (
-                    root.find(SCPD_DEVICE).find(SCPD_PRESENTATIONURL).text)
-                device["modelName"] = (
-                    root.find(SCPD_DEVICE).find(SCPD_MODELNAME).text)
-                device["serialNumber"] = (
-                    root.find(SCPD_DEVICE).find(SCPD_SERIALNUMBER).text)
-                device["friendlyName"] = (
-                    root.find(SCPD_DEVICE).find(SCPD_FRIENDLYNAME).text)
-                return device
+                    device_xml.find(SCPD_PRESENTATIONURL).text)
             else:
-                return False
+                device["host"] = urlparse(url).hostname
+            device["modelName"] = (
+                device_xml.find(SCPD_MODELNAME).text)
+            device["serialNumber"] = (
+                device_xml.find(SCPD_SERIALNUMBER).text)
+            device["friendlyName"] = (
+                device_xml.find(SCPD_FRIENDLYNAME).text)
+            return device
         except (AttributeError, ValueError, ET.ParseError) as err:
             _LOGGER.error(
                 "Error occurred during evaluation of SCPD XML: %s", err)
